'use strict';

function getTypedEntries(object) {
    return Object.entries(object);
}

function clone(unknownValue) {
    if (!unknownValue) {
        return unknownValue;
    }
    else if (typeof unknownValue !== "object") {
        return unknownValue;
    }
    else if (unknownValue instanceof Array) {
        return unknownValue.map(clone);
    }
    else {
        return getTypedEntries(unknownValue).reduce((pv, [key, value]) => {
            pv[key] = clone(value);
            return pv;
        }, {});
    }
}

function entryUseMapper([key, value], mapper) {
    const ReTypedMapper = mapper;
    return ReTypedMapper[key](value);
}

function escapeRegExp(text) {
    return text.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
}

function getTypedKeys(object) {
    return Object.keys(object);
}

function hasKey(obj, key) {
    return key in obj;
}

function createInterpolation(value, _strict) {
    return (interpolationValues) => (interpolationValues
        ? value.replace(/\{([^}]*)\}/g, (match, interpolationId) => interpolationValues[interpolationId])
        : value);
}

function simpleClone(unknownValue) {
    if (!unknownValue) {
        return unknownValue;
    }
    else if (typeof unknownValue !== "object") {
        return unknownValue;
    }
    else if (unknownValue.constructor?.name === "Object"
        || unknownValue.constructor === undefined) {
        return getTypedEntries(unknownValue).reduce((pv, [key, value]) => {
            pv[key] = simpleClone(value);
            return pv;
        }, {});
    }
    else if (unknownValue instanceof Array && unknownValue.constructor.name === "Array") {
        return unknownValue.map(simpleClone);
    }
    else {
        return unknownValue;
    }
}

function sleep(milliseconde) {
    return new Promise((resolve) => void setTimeout(resolve, milliseconde));
}

/* eslint-disable id-length */
class InvalidBytesInStringError extends Error {
    input;
    constructor(input) {
        super("Invalid Input");
        this.input = input;
    }
}
const parseRegExp = /^((-|\+)?(\d+(?:\.\d+)?)) *(kb|mb|gb|tb|pb)$/i;
const unitMapper = {
    b: 1,
    kb: 1 << 10,
    mb: 1 << 20,
    gb: 1 << 30,
    tb: Math.pow(1024, 4),
    pb: Math.pow(1024, 5),
};
function stringToBytes(bytesInString) {
    if (typeof bytesInString === "number") {
        return bytesInString;
    }
    const regExpResults = parseRegExp.exec(bytesInString);
    const floatValue = regExpResults
        ? parseFloat(regExpResults[1])
        : parseInt(bytesInString, 10);
    const unit = regExpResults
        ? regExpResults[4].toLowerCase()
        : "b";
    if (!hasKey(unitMapper, unit) || isNaN(floatValue)) {
        throw new InvalidBytesInStringError(bytesInString);
    }
    return Math.floor(unitMapper[unit] * floatValue);
}

class UnPartialError extends Error {
    key;
    partialObject;
    constructor(key, partialObject) {
        super(`The key "${key.toString()}" is missing in partial object.`);
        this.key = key;
        this.partialObject = partialObject;
    }
}
function unPartial(partialObject, keys) {
    keys.forEach((key) => {
        if (partialObject[key] === undefined) {
            throw new UnPartialError(key, partialObject);
        }
    });
    return partialObject;
}

exports.InvalidBytesInStringError = InvalidBytesInStringError;
exports.UnPartialError = UnPartialError;
exports.clone = clone;
exports.createInterpolation = createInterpolation;
exports.entryUseMapper = entryUseMapper;
exports.escapeRegExp = escapeRegExp;
exports.getTypedEntries = getTypedEntries;
exports.getTypedKeys = getTypedKeys;
exports.hasKey = hasKey;
exports.simpleClone = simpleClone;
exports.sleep = sleep;
exports.stringToBytes = stringToBytes;
exports.unPartial = unPartial;
